// LED clock with 8x8 LED matrix
// Details on:
// www.homofaciens.de/technics-physical-computing-LED-matrix_en.htm


// Define the pin numbers for the rows
#define ROW_1        8
#define ROW_2       11
#define ROW_3       12
#define ROW_4       13
#define ROW_5       A0
#define ROW_6       A1
#define ROW_7       A2
#define ROW_8       A3

// Define the pin numbers for the columns
#define COL_1         7
#define COL_2         6
#define COL_3         5
#define COL_4         4
#define COL_5         3
#define COL_6         2
#define COL_7         1
#define COL_8         0

#define BUZZER        9
#define MIC          A5

#define SWITCH_SEC   10
#define SWITCH_MIN   A5
#define SWITCH_HOUR  A4

#define LED_HOUR     60 
#define LED_MIN      61 
#define LED_SEC      62 
#define LED_AM       63

// Define number of columns and rows
#define SIZE_COLS     8
#define SIZE_ROWS     8

#define ON_DELAY    100

int hour_led;
int minute_led;
int second_led;

// Define the arrays of the pin numbers of Rows and columns
int rows[SIZE_ROWS] = {ROW_1, ROW_2, ROW_3, ROW_4, ROW_5, ROW_6, ROW_7, ROW_8};
int cols[SIZE_COLS] = {COL_1, COL_2, COL_3, COL_4, COL_5, COL_6, COL_7, COL_8};

int matrix_hour[SIZE_ROWS] = {0,0,0,0,0,0,0,0};
int matrix_min[SIZE_ROWS] = {0,0,0,0,0,0,0,0};
int matrix_sec[SIZE_ROWS] = {0,0,0,0,0,0,0,0};


int rowCalc = 0;
int temp = second_led;
int alarm, toneNow;
unsigned char a, b;
unsigned long timeCounter;
int loopCount;
int ButtonSecState;
int ButtonMinState;
int ButtonHourState;
int ledSecOn;
int ledMinOn;
int ledHourOn;
int ledAmPm;
int minutesBlinkTime;

int i, j;

int melodie[52] = {602,676,758,1012,   758,602,676,1012, 758,676,602,758,   602,758,676,1012, 1012,676,602,758, 602,676,758,1012,   758,602,676,1012, 758,676,602,758, 602,758,676,1012, 1012,676,602,758, 758,758,758,758,758,758,758,758,758,758,758,758};
int melodieStart, melodieEnd;

// Setup routine runs at startup or when you press reset:
void setup() {
  // initialize the digital pin as an output.
  for(i=0; i < SIZE_ROWS; i++){
    pinMode(rows[i], OUTPUT);
    // To turn OFF all rows, set them to HIGH
    digitalWrite(rows[i], HIGH);
  }
  for(i=0; i < SIZE_COLS; i++){
    pinMode(cols[i], OUTPUT);
    // To turn OFF all columns, set them to LOW
    digitalWrite(cols[i], LOW);
  }

  pinMode(BUZZER, OUTPUT);

  // Turn on pull-up resistors
  digitalWrite(SWITCH_SEC, HIGH);
  digitalWrite(SWITCH_MIN, HIGH);
  digitalWrite(SWITCH_HOUR, HIGH);

  second_led = 0;
  minute_led = 59;
  hour_led = 9;
  timeCounter = 0;
  loopCount = 0;
  alarm = 0;
  melodieStart = 12;
  melodieEnd = 23;
  toneNow = melodieStart;

  ButtonSecState = 0;
  ButtonMinState = 0;
  ButtonHourState = 0;
  ledSecOn = 0;
  ledMinOn = 0;
  ledHourOn = 0;
  ledAmPm = 0;
  minutesBlinkTime = 100;
}

// the loop routine runs over and over again forever:
void loop() {

  loopCount = 0;
  while(millis()-timeCounter < 1000UL){  
    for(int row=0; row<SIZE_ROWS; row++){
      a = matrix_sec[row] | matrix_hour[row];
      if(loopCount < minutesBlinkTime){
        a |= matrix_min[row]; 
      }
      if(loopCount > minutesBlinkTime * 2){
        loopCount = 0;
      }
      if(row == 7 && ledSecOn == 1){
        a |= 0b00001000; 
      }
      if(row == 7 && ledMinOn == 1){
        a |= 0b00000100; 
      }
      if(row == 7 && ledHourOn == 1){
        a |= 0b00000010; 
      }
      if(row == 7 && ledAmPm == 1){
        a |= 0b00000001; 
      }
      PORTD = a;
      digitalWrite(rows[row], 0);
      
      // Check time setting switsches
      if(digitalRead(SWITCH_SEC) == 0){
        if(ButtonSecState == 0){
          second_led = -1;
          ButtonSecState = 10;
          ledSecOn = 1;
        }
        else{
          ButtonSecState--;
        }
      }
      else{
        if(ButtonSecState > 0){
          ButtonSecState-=1;
        }
        else{
          ledSecOn = 0;
        }
      }
      
      if(digitalRead(SWITCH_MIN) == 0){
        if(ButtonMinState == 0){
          minute_led++;
          if(minute_led == 60){
            minute_led = 0;
          }
          ButtonMinState = 1000;
          ledMinOn = 1;
        }
        else{
          if(ButtonMinState > 0){
            ButtonMinState--;
          }
        }
      }
      else{
        if(ButtonMinState > 99){
          ButtonMinState-=100;
        }
        else{
          ledMinOn = 0;
          ButtonMinState=0;
        }
      }
    
      if(digitalRead(SWITCH_HOUR) == 0){
        if(ButtonHourState == 0){
          hour_led++;
          if(hour_led == 12){
            hour_led = 0;
          }
          ButtonHourState = 1000;
          ledHourOn = 1;
        }
        else{
          if(ButtonHourState > 0){
            ButtonHourState--;
          }
        }
      }
      else{
        if(ButtonHourState > 99){
          ButtonHourState-=100;
        }
        else{
          ButtonHourState=0;
          ledHourOn = 0;
        }
      }
          

      if(alarm == 0){
        delay(1);
      }
      else{
        // Play Westminster melodie
        digitalWrite(BUZZER, HIGH);
        delayMicroseconds(melodie[toneNow]/10);
        digitalWrite(BUZZER, LOW);
        delayMicroseconds(melodie[toneNow]*3);
      }
      digitalWrite(rows[row], 1);
    }
    loopCount++;
  }

  if(alarm == 1){
    toneNow++;
    if(toneNow == melodieEnd+1){
      toneNow = melodieStart;
      alarm = 0;
      minutesBlinkTime = 100;
    }
    if(toneNow > 39){
      delay(200);
    }
  }
  timeCounter+=1000UL;
  second_led++;
  if(second_led == 60){
    second_led = 0;
    minute_led++;
  }
  if(minute_led == 15 && second_led == 0){
    alarm = 1;
    minutesBlinkTime = 50;
    melodieStart = 0;
    melodieEnd = 3;
    toneNow = melodieStart;
  }
  if(minute_led == 30 && second_led == 0){
    alarm = 1;
    minutesBlinkTime = 50;
    melodieStart = 4;
    melodieEnd = 11;
    toneNow = melodieStart;
  }
  if(minute_led == 45 && second_led == 0){
    alarm = 1;
    minutesBlinkTime = 50;
    melodieStart = 12;
    melodieEnd = 23;
    toneNow = melodieStart;
  }
  if(minute_led == 60 && second_led == 0){
    minute_led = 0;
    hour_led++;
    alarm = 1;
    minutesBlinkTime = 50;
    melodieStart = 24;
    melodieEnd = 39 + hour_led;
    if(hour_led == 0){
      melodieEnd += 12;
    }
    toneNow = melodieStart;
  }
  if(hour_led == 12){
    hour_led = 0;
    if(ledAmPm == 0){
      ledAmPm = 1;
    }
    else{
      ledAmPm = 0;
    }
  }

          

rowCalc = 0;
temp = hour_led * 5 + minute_led / 12 - 1;
if(temp == -1){
  temp = 59;
}
while(temp > 7){
  temp -= 8;
  rowCalc++;
}
for(i=0; i < 8; i++){
  matrix_hour[i] = 0;
}
matrix_hour[rowCalc] = 1 << (7-temp);

rowCalc = 0;
temp = minute_led-1;
if(temp == -1){
  temp = 59;
}
while(temp > 7){
  temp -= 8;
  rowCalc++;
}
for(i=0; i < 8; i++){
  matrix_min[i] = 0;
}
matrix_min[rowCalc] = 1 << (7-temp);

rowCalc = 0;
temp = second_led-1;
if(temp == -1){
  temp = 59;
}
while(temp > 7){
  temp -= 8;
  rowCalc++;
}
for(i=0; i < 8; i++){
  matrix_sec[i] = 0;
}
matrix_sec[rowCalc] = 1 << (7-temp);



/*  
  //Loop through all LEDs on the clock
  for(int row=0; row < SIZE_ROWS; row++){
    digitalWrite(rows[row], 0);
    for(int col=0; col < SIZE_COLS; col++){
      digitalWrite(cols[col], 1);  
      delay(ON_DELAY);
      digitalWrite(cols[col], 0);
    }
    digitalWrite(rows[row], 1);
  }
*/



}
